@@ -1,15 +1,12 @@
-# -*- coding: utf-8 -*-
 """
 Module to hold Eclipse input and output filenames
 """
-from __future__ import print_function
-from __future__ import absolute_import
-from __future__ import division
 
 import os
 import errno
 import logging
 import shlex
+from pathlib import Path
 
 import opm.io
 
@@ -17,7 +14,6 @@ from ecl.eclfile import EclFile
 from ecl.grid import EclGrid
 from ecl.summary import EclSum
 
-logging.basicConfig()
 logger = logging.getLogger(__name__)
 
 # Default parse option to opm.io for a very permissive parsing
@@ -34,12 +30,6 @@ OPMIOPARSER_RECOVERY = [
     ("PARSE_EXTRA_DATA", opm.io.action.ignore),
 ]
 
-# For Python2 compatibility:
-try:
-    FileNotFoundError
-except NameError:
-    FileNotFoundError = IOError
-
 
 class EclFiles(object):
     """
@@ -59,7 +49,7 @@ class EclFiles(object):
         eclbase = str(eclbase)
 
         # Hint about possible wrong filenames:
-        if ".DATA" in eclbase and not os.path.exists(eclbase):
+        if ".DATA" in eclbase and not Path(eclbase).is_file():
             logger.warning("File %s does not exist", eclbase)
             # (this is not an error, because it is possible
             # to obtain summary without the DATA file being present)
@@ -83,12 +73,12 @@ class EclFiles(object):
 
     def get_path(self):
         """Return the full path to the directory with the DATA file"""
-        return os.path.dirname(os.path.abspath(self._eclbase))
+        return Path(self._eclbase).absolute().parent
 
     def get_ecldeck(self):
         """Return a opm.io deck of the DATA file"""
         if not self._deck:
-            if os.path.exists(self._eclbase + ".DATA"):
+            if Path(self._eclbase + ".DATA").is_file():
                 deckfile = self._eclbase + ".DATA"
             else:
                 deckfile = self._eclbase  # Will be any filename
@@ -117,7 +107,7 @@ class EclFiles(object):
         """Find and return EGRID file as an EclGrid object"""
         if not self._egrid:
             egridfilename = self._eclbase + ".EGRID"
-            if not os.path.exists(egridfilename):
+            if not Path(egridfilename).is_file():
                 raise FileNotFoundError(
                     errno.ENOENT, os.strerror(errno.ENOENT), egridfilename
                 )
@@ -131,7 +121,7 @@ class EclFiles(object):
         This gives access to data vectors defined on the grid."""
         if not self._egridfile:
             egridfilename = self._eclbase + ".EGRID"
-            if not os.path.exists(egridfilename):
+            if not Path(egridfilename).is_file():
                 raise FileNotFoundError(
                     errno.ENOENT, os.strerror(errno.ENOENT), egridfilename
                 )
@@ -151,7 +141,7 @@ class EclFiles(object):
         """
         if not self._eclsum:
             smryfilename = self._eclbase + ".UNSMRY"
-            if not os.path.exists(smryfilename):
+            if not Path(smryfilename).is_file():
                 raise FileNotFoundError(
                     errno.ENOENT, os.strerror(errno.ENOENT), smryfilename
                 )
@@ -163,7 +153,7 @@ class EclFiles(object):
         """Find and return the INIT file as an EclFile object"""
         if not self._initfile:
             initfilename = self._eclbase + ".INIT"
-            if not os.path.exists(initfilename):
+            if not Path(initfilename).is_file():
                 raise FileNotFoundError(
                     errno.ENOENT, os.strerror(errno.ENOENT), initfilename
                 )
@@ -175,7 +165,7 @@ class EclFiles(object):
         """Find and return the RFT file as an EclFile object"""
         if not self._rftfile:
             rftfilename = self._eclbase + ".RFT"
-            if not os.path.exists(rftfilename):
+            if not Path(rftfilename).is_file():
                 raise FileNotFoundError(
                     errno.ENOENT, os.strerror(errno.ENOENT), rftfilename
                 )
@@ -187,7 +177,7 @@ class EclFiles(object):
         """Find and return the UNRST file as an EclFile object"""
         if not self._rstfile:
             rstfilename = self._eclbase + ".UNRST"
-            if not os.path.exists(rstfilename):
+            if not Path(rstfilename).is_file():
                 raise FileNotFoundError(
                     errno.ENOENT, os.strerror(errno.ENOENT), rstfilename
                 )
@@ -232,11 +222,11 @@ class EclFiles(object):
         else:
             filename_defaulted = False
         assert isinstance(filename, str)
-        if not os.path.isabs(filename):
-            fullpath = os.path.join(self.get_path(), filename)
+        if not Path(filename).is_absolute():
+            fullpath = Path(self.get_path()) / filename
         else:
             fullpath = filename
-        if not os.path.exists(fullpath):
+        if not Path(fullpath).is_file():
             if filename_defaulted:
                 # No warnings when the default filename is not there.
                 return {}
@@ -245,19 +235,26 @@ class EclFiles(object):
 
         zonelines = open(fullpath).readlines()
         zonelines = [line.strip() for line in zonelines]
-        zonelines = [line for line in zonelines if not line.startswith("--")]
+        zonelines = [line.split("--")[0] for line in zonelines]
         zonelines = [line for line in zonelines if not line.startswith("#")]
         zonelines = filter(len, zonelines)
 
         zonemap = {}
         for line in zonelines:
-            (layername, interval) = shlex.split(line)
-            (k_0, k_1) = interval.strip().split("-")
-            for k_idx in range(int(k_0), int(k_1) + 1):
-                zonemap[k_idx] = layername
+            try:
+                linesplit = shlex.split(line)
+                map(str.strip, linesplit)
+                filter(len, linesplit)
+                (k_0, k_1) = "".join(linesplit[1:]).split("-")
+                for k_idx in range(int(k_0), int(k_1) + 1):
+                    zonemap[k_idx] = linesplit[0]
+            except ValueError:
+                logger.error("Could not parse zonemapfile %s", filename)
+                logger.error("Failed on content: %s", line)
+                return
         return zonemap
 
 
 def rreplace(pat, sub, string):
     """Variant of str.replace() that only replaces at the end of the string"""
-    return string[0 : -len(pat)] + sub if string.endswith(pat) else string
+    return string[0 : -len(pat)] + sub if string.endswith(pat) else string  # noqa
