@@ -1,20 +1,17 @@
 """Test module for fipreports"""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
-import os
 import sys
+from pathlib import Path
 
 import pandas as pd
 
 from ecl2df import fipreports, ecl2csv
 from ecl2df.eclfiles import EclFiles
+from ecl2df.fipreports import report_block_lineparser as parser
 
-TESTDIR = os.path.dirname(os.path.abspath(__file__))
-DATAFILE = os.path.join(TESTDIR, "data/reek/eclipse/model/2_R001_REEK-0.DATA")
-MOCKPRTFILE = os.path.join(TESTDIR, "data/fipreports/TEST1.PRT")
+TESTDIR = Path(__file__).absolute().parent
+DATAFILE = str(TESTDIR / "data/reek/eclipse/model/2_R001_REEK-0.DATA")
+MOCKPRTFILE = str(TESTDIR / "data/fipreports/TEST1.PRT")
 
 
 def test_fipreports2df():
@@ -52,7 +49,6 @@ def test_report_block_lineparser():
     """
     Test the line-parser, which has to infer partly which phases are present.
     """
-    from ecl2df.fipreports import report_block_lineparser as parser
 
     tup = parser(
         " :OUTFLOW THROUGH WELLS    :                                         0.:"
@@ -85,11 +81,11 @@ def test_report_block_lineparser():
 
 def test_cmdline(tmpdir):
     """Test command line interface"""
-    tmpcsvfile = tmpdir.join(".TMP-fipreports.csv")
+    tmpcsvfile = tmpdir / "TMP-fipreports.csv"
     sys.argv = ["ecl2csv", "fipreports", "-v", DATAFILE, "--output", str(tmpcsvfile)]
     ecl2csv.main()
 
-    assert os.path.exists(str(tmpcsvfile))
+    assert Path(tmpcsvfile).is_file()
     disk_df = pd.read_csv(tmpcsvfile)
     assert "FIPNAME" in disk_df
     assert "STOIIP_OIL" in disk_df
