@@ -1,13 +1,9 @@
 """Test module for rft"""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
-import os
 import sys
 import random
 import datetime
+from pathlib import Path
 
 import pandas as pd
 import numpy as np
@@ -17,8 +13,10 @@ import pytest
 from ecl2df import rft, ecl2csv
 from ecl2df.eclfiles import EclFiles
 
-TESTDIR = os.path.dirname(os.path.abspath(__file__))
-DATAFILE = os.path.join(TESTDIR, "data/reek/eclipse/model/2_R001_REEK-0.DATA")
+TESTDIR = Path(__file__).absolute().parent
+DATAFILE = str(TESTDIR / "data/reek/eclipse/model/2_R001_REEK-0.DATA")
+
+# pylint: disable=protected-access
 
 
 def test_rftrecords2df():
@@ -53,6 +51,7 @@ def test_rftrecords_generator():
 
 
 def test_get_con_seg_data():
+    """Get CON data. Later add more code here to defend the name"""
     rftfile = EclFiles(DATAFILE).get_rftfile()
 
     # Test the first record, it is a CON type (not multisegment)
@@ -70,7 +69,7 @@ def test_get_con_seg_data():
 def test_minimal_well():
     """Test a dummy well dataset
 
-        |    segidx 1
+    |    segidx 1
 
     """
     one_seg = pd.DataFrame(
@@ -97,8 +96,8 @@ def test_minimal_well():
 
 def test_minimal_branched_well():
     r"""
-        |       segidx 1
-       / \      segidx 2 and 3
+     |       segidx 1
+    / \      segidx 2 and 3
     """
     two_branch = pd.DataFrame(
         {"SEGIDX": [1, 2, 3], "SEGNXT": [None, 1, 1], "SEGBRNO": [1, 1, 2]}
@@ -224,8 +223,8 @@ def test_single_branch_partly_icd():
 def test_branched_icd_well():
     r"""Simplest possible branched well with ICD segments
 
-           |          segidx 1
-      * - / \ - *     segidx 2 and 3
+         |          segidx 1
+    * - / \ - *     segidx 2 and 3
     """
     wellseg = pd.DataFrame(
         {
@@ -362,7 +361,7 @@ def test_longer_branched_partly_icd_well():
 def test_rft2df():
     """Test that dataframes are produced"""
     eclfiles = EclFiles(DATAFILE)
-    rftdf = rft.rft2df(eclfiles)
+    rftdf = rft.df(eclfiles)
     assert "ZONE" in rftdf
     assert "LEAF" not in rftdf  # Topology metadata should not be exported
     assert set(rftdf["WELLMODEL"]) == {"STANDARD"}
@@ -383,28 +382,17 @@ def test_rft2df():
     assert not rftdf.columns.empty
 
 
-def test_main(tmpdir):
-    """Test command line interface"""
-    tmpcsvfile = tmpdir.join(".TMP-rft.csv")
-    sys.argv = ["rft2csv", DATAFILE, "-o", str(tmpcsvfile)]
-    rft.main()
-
-    assert os.path.exists(str(tmpcsvfile))
-    disk_df = pd.read_csv(str(tmpcsvfile))
-    assert not disk_df.empty
-
-
 def test_main_subparsers(tmpdir):
     """Test command line interface"""
-    tmpcsvfile = tmpdir.join(".TMP-rft.csv")
+    tmpcsvfile = tmpdir / ".TMP-rft.csv"
     sys.argv = ["ecl2csv", "rft", DATAFILE, "-o", str(tmpcsvfile)]
     ecl2csv.main()
 
-    assert os.path.exists(str(tmpcsvfile))
+    assert Path(tmpcsvfile).is_file()
     disk_df = pd.read_csv(str(tmpcsvfile))
     assert not disk_df.empty
 
-    tmpcsvfile = tmpdir.join(".TMP-rft2.csv")
+    tmpcsvfile = tmpdir / ".TMP-rft2.csv"
     # Test with RFT file as argument:
     sys.argv = [
         "ecl2cvsv",
@@ -415,6 +403,6 @@ def test_main_subparsers(tmpdir):
         str(tmpcsvfile),
     ]
     ecl2csv.main()
-    assert os.path.exists(str(tmpcsvfile))
+    assert Path(tmpcsvfile).is_file()
     disk_df = pd.read_csv(str(tmpcsvfile))
     assert not disk_df.empty
