@@ -1,11 +1,13 @@
 """Test module for satfunc2df"""
 
-import sys
+import subprocess
 from pathlib import Path
 
+import pytest
+
 import pandas as pd
 
-from ecl2df import satfunc, ecl2csv, inferdims
+from ecl2df import satfunc, ecl2csv, csv2ecl, inferdims
 from ecl2df.eclfiles import EclFiles
 
 TESTDIR = Path(__file__).absolute().parent
@@ -38,6 +40,23 @@ def test_satfunc2df():
     )
 
 
+def test_df2ecl_order():
+    """Test that we can control the keyword order in generated
+    strings by the list supplied in keywords argument"""
+    eclfiles = EclFiles(DATAFILE)
+    satdf = satfunc.df(eclfiles.get_ecldeck())
+
+    swof_sgof = satfunc.df2ecl(satdf, keywords=["SWOF", "SGOF"])
+    assert swof_sgof.find("SWOF") < swof_sgof.find("SGOF")
+    sgof_swof = satfunc.df2ecl(satdf, keywords=["SGOF", "SWOF"])
+    assert sgof_swof.find("SGOF") < sgof_swof.find("SWOF")
+
+    only_swof = satfunc.df2ecl(satdf, keywords=["SWOF"])
+    assert "SGOF" not in only_swof
+    only_sgof = satfunc.df2ecl(satdf, keywords="SGOF")
+    assert "SWOF" not in only_sgof
+
+
 def test_nodata():
     """Test when no data is found"""
     swofstr = ""
@@ -208,7 +227,7 @@ def test_sgwfn():
     )
 
 
-def test_sgof_satnuminferrer(tmpdir):
+def test_sgof_satnuminferrer(tmpdir, mocker):
     """Test inferring of SATNUMS in SGOF strings"""
     sgofstr = """
 SGOF
@@ -236,19 +255,81 @@ SGOF
 
     # Write to file and try to parse it with command line:
     sgoffile = "__sgof_tmp.txt"
-    with open(sgoffile, "w") as sgof_f:
-        sgof_f.write(sgofstr)
-
-    sys.argv = ["ecl2csv", "satfunc", "-v", sgoffile, "-o", sgoffile + ".csv"]
+    Path(sgoffile).write_text(sgofstr)
+    mocker.patch(
+        "sys.argv", ["ecl2csv", "satfunc", "-v", sgoffile, "-o", sgoffile + ".csv"]
+    )
     ecl2csv.main()
     parsed_sgof = pd.read_csv(sgoffile + ".csv")
     assert len(parsed_sgof["SATNUM"].unique()) == 3
 
 
-def test_main_subparsers(tmpdir):
+def test_wrong_columns():
+    """Test some error situations"""
+    # SWFN data given as SWOF:
+    satnumstr = """
+SWOF
+0 0 0
+1 1 0
+/
+"""
+    with pytest.raises(ValueError, match="Wrong number count for keyword SWOF"):
+        satfunc.df(satnumstr)
+    satnumstr = """
+SWFN
+0 0 0 0
+1 1 0 0
+/
+"""
+    with pytest.raises(ValueError, match="Wrong number count for keyword SWFN"):
+        satfunc.df(satnumstr)
+
+    # The following error is parseable into a dataframe, but gives
+    # four saturation points, this error can not be detected while parsing.
+    satnumstr = """
+SWFN
+0 0 0 0
+0.5 0.5 0.5 0
+1 1 0 0
+/
+"""
+    wrongdf = satfunc.df(satnumstr)
+    # We see the error as the saturation points are not unique:
+    assert len(wrongdf["SW"]) == 4
+    assert len(wrongdf["SW"].unique()) == 3
+
+
+def test_multiple_keywords_family2():
+
+    satnumstr = """
+SWFN
+-- Sw           Krw           Pcow
+  0 0 2
+  1.   1.000   0.00000e+00
+/
+
+SOF3
+-- So           Krow          Krog
+   0.00000e+00   0.00000e+00   0.00000e+00
+   0.581051658   1.000000000   1.000000000
+/
+
+SGFN
+-- Sg    Krg      Pcog
+  0.000  0.00000  0.000
+  0.800  1.00000  0.000
+/
+    """
+    satnum_df = satfunc.df(satnumstr)
+    assert set(satnum_df["SATNUM"]) == {1}
+    assert set(satnum_df["KEYWORD"]) == {"SWFN", "SOF3", "SGFN"}
+    assert len(satnum_df) == 6
+
+
+def test_main_subparsers(tmpdir, mocker):
     """Test command line interface"""
-    tmpcsvfile = tmpdir.join(".TMP-satfunc.csv")
-    sys.argv = ["ecl2csv", "satfunc", DATAFILE, "-o", str(tmpcsvfile)]
+    tmpcsvfile = tmpdir.join("satfunc.csv")
+    mocker.patch("sys.argv", ["ecl2csv", "satfunc", DATAFILE, "-o", str(tmpcsvfile)])
     ecl2csv.main()
 
     assert Path(tmpcsvfile).is_file()
@@ -257,17 +338,49 @@ def test_main_subparsers(tmpdir):
 
     tmpcsvfile2 = tmpdir.join(".TMP-satfunc-swof.csv")
     print(tmpcsvfile2)
-    sys.argv = [
-        "ecl2csv",
-        "satfunc",
-        DATAFILE,
-        "--keywords",
-        "SWOF",
-        "--output",
-        str(tmpcsvfile2),
-    ]
+    mocker.patch(
+        "sys.argv",
+        [
+            "ecl2csv",
+            "satfunc",
+            DATAFILE,
+            "--keywords",
+            "SWOF",
+            "--output",
+            str(tmpcsvfile2),
+        ],
+    )
     ecl2csv.main()
 
     assert Path(tmpcsvfile2).is_file()
     disk_df = pd.read_csv(str(tmpcsvfile2))
     assert set(disk_df["KEYWORD"].unique()) == {"SWOF"}
+
+
+def test_csv2ecl(tmpdir, mocker):
+    """Test command line interface for csv to Eclipse include files"""
+    tmpdir.chdir()
+    tmpcsvfile = "satfunc.csv"
+
+    swof_df = pd.DataFrame(
+        columns=["KEYWORD", "SW", "KRW", "KROW", "PCOW"],
+        data=[["SWOF", 0.0, 0.0, 1.0, 0.0], ["SWOF", 1.0, 1.0, 0.0, 0.0]],
+    )
+    swof_df.to_csv(tmpcsvfile, index=False)
+    mocker.patch("sys.argv", ["csv2ecl", "satfunc", "--output", "swof.inc", tmpcsvfile])
+    csv2ecl.main()
+    pd.testing.assert_frame_equal(
+        satfunc.df(open("swof.inc").read()).drop("SATNUM", axis="columns"),
+        swof_df,
+        check_like=True,
+    )
+
+    # Test writing to stdout:
+    result = subprocess.run(
+        ["csv2ecl", "satfunc", "--output", "-", tmpcsvfile], stdout=subprocess.PIPE
+    )
+    pd.testing.assert_frame_equal(
+        satfunc.df(result.stdout.decode()).drop("SATNUM", axis="columns"),
+        swof_df,
+        check_like=True,
+    )
